/*
   Dico

   Luc Maranget, projet Moscova, Inria Rocquencourt
 
   Copyright 2005 Institut National de Recherche en Informatique et    
   en Automatique.  All rights reserved.  This file is distributed     
   under the terms of the Q Public License version 1.0.                
*/

import java.util.* ;

class SimpleHash {

  final static char NOCHAR = (char)-1 ;

  private static char hash(char key) {
    key += ~(key << 7);
    key ^= (key >>> 1);
    key += (key << 2);
    key ^=  (key >>> 7);
    key += ~(key << 4);
    key ^=  (key >>> 8);
    return key;
  }
  
  private final static int NSTART = 2 ;
  private char [] t = new char[NSTART] ;
  private char [] me = new char[NSTART] ;
  
  private char next = 0 ;

  private void doAdd (char c, char v) {
    char h = hash(c) ;
    int len = me.length ;

    for ( ;; ) {
      h %= len ;
      if (me[h] == NOCHAR) {
        me[h] = c ; t[h] = v ;
        return ;
      }
      h++ ;
    }
  }

  char get(char c) {
    int len = me.length ;
    char h = hash(c) ;
    for ( ;; ) {
      h %= len ;
      if (me[h] == NOCHAR) {
        return NOCHAR ;
      } else if (me[h] == c) {
        return t[h] ;
      } else {
        h++ ;
      }
    }
  }

  boolean look(char c) {
    return get(c) != NOCHAR ;
  }

  void dump() {
    System.err.println("==============================") ;
    for (char i = 0 ; i < me.length ; i++) {
      char x = me[i] ;
      if (x == NOCHAR)
        System.err.println( ((int) i) + ":") ;
      else
        System.err.println( ((int) i) + ": " + x + ", "+  (int)t[i]) ;
    }
  }

   void add(char c) {
    if (look(c)) return ;

    int len = me.length ;

    if (3 * next >= len) {
      resize() ;
    }
    doAdd(c,next) ;
    next++ ;
  }

  private void init_me() {
    int len = me.length ;

    for (char i = 0 ; i < len ; i++) {
      me[i] = NOCHAR ;
    }
  }


  private void resize() {
    int old = t.length ;
    char [] new_t = new char [2*old] ;
    char [] new_me = new char [2*old] ;
    char [] old_t = t ;
    char [] old_me = me ;

    t = new_t ; me = new_me ;
    init_me() ;

    for (char i = 0 ; i < old ; i++) {
      if (old_me[i] != NOCHAR) {
        doAdd(old_me[i], old_t[i]);
      }
    }
  }

   char [] inverse() {
    char [] r = new char[next] ;
    int len = me.length ;

    for (int i = 0 ; i < len ; i++) {
      char c = me[i] ;
      if (c != NOCHAR) {
        r[t[i]] = c ;
      }
    }
    return r ;
  }

   SimpleHash() {
    init_me() ;
    add('\0') ;
  }

   char code(char c) {
    int i = get(c) ;
    //    System.err.println(c + " -> " + i) ;
    return (char)i ;
  }

  public static void main(String [] arg) {
    SimpleHash t = new SimpleHash () ;
    for (int i = 0 ; i < arg[0].length() ; i++)
      t.add(arg[0].charAt(i)) ;
    t.dump() ;
  }
}
